package api

import (
	"strconv"

	"github.com/astaxie/beego"
	"github.com/deluan/gosonic/api/responses"
	"github.com/deluan/gosonic/domain"
	"github.com/deluan/gosonic/engine"
	"github.com/deluan/gosonic/utils"
	"github.com/karlkfi/inject"
)

type StreamController struct {
	BaseAPIController
	repo domain.MediaFileRepository
	id   string
	mf   *domain.MediaFile
}

func (c *StreamController) Prepare() {
	inject.ExtractAssignable(utils.Graph, &c.repo)

	c.id = c.RequiredParamString("id", "id parameter required")

	mf, err := c.repo.Get(c.id)
	if err != nil {
		beego.Error("Error reading mediafile", c.id, "from the database", ":", err)
		c.SendError(responses.ERROR_GENERIC, "Internal error")
	}

	if mf == nil {
		beego.Error("MediaFile", c.id, "not found!")
		c.SendError(responses.ERROR_DATA_NOT_FOUND)
	}

	c.mf = mf
}

// TODO Still getting the "Conn.Write wrote more than the declared Content-Length" error.
// Don't know if this causes any issues
func (c *StreamController) Stream() {
	maxBitRate := c.ParamInt("maxBitRate")
	maxBitRate = utils.MinInt(c.mf.BitRate, maxBitRate)

	beego.Debug("Streaming file", c.id, ":", c.mf.Path)
	beego.Debug("Bitrate", c.mf.BitRate, "MaxBitRate", maxBitRate)

	contentLength := c.mf.Size
	if maxBitRate > 0 {
		contentLength = strconv.Itoa((c.mf.Duration + 1) * maxBitRate * 1000 / 8)
	}
	c.Ctx.Output.Header("Content-Length", contentLength)
	c.Ctx.Output.Header("Content-Type", "audio/mpeg")
	c.Ctx.Output.Header("Expires", "0")
	c.Ctx.Output.Header("Cache-Control", "must-revalidate")
	c.Ctx.Output.Header("Pragma", "public")

	err := engine.Stream(c.mf.Path, c.mf.BitRate, maxBitRate, c.Ctx.ResponseWriter)
	if err != nil {
		beego.Error("Error streaming file", c.id, ":", err)
	}

	beego.Debug("Finished streaming of", c.mf.Path)
}

func (c *StreamController) Download() {
	beego.Debug("Sending file", c.mf.Path)

	engine.Stream(c.mf.Path, 0, 0, c.Ctx.ResponseWriter)

	beego.Debug("Finished sending", c.mf.Path)
}
